<?php
/*
Plugin Name: Favatars-Revised
Plugin URI: http://dev.wp-plugins.org/wiki/favatars
Description: A system to show favicon.ico files as avatars - "Favatars". Entire recode by <a href="http://www.thecodepro.com/">Jeff Minard</a><br />Previous Versions care of: <a href="http://www.noscope.com">Joen</a>, <a href="http://www.peej.co.uk/projects/favatars.html">Paul James</a>, and <a href="http://www.somefoolwitha.com">Matthew</a>. 
Version: 1.0.2
Author: Jeff Minard
Author URI: http://thecodepro.com/
*/

/*



**** INSTALLATION *****

1. Drop this file into your plugins directory (wp-content/plugins).
2. Activate the plugin on your WordPress Plugin Admin page.
3. Add `<?php comment_favicon(); ?>` somewhere inside the comment loop.




************************ 
****** IMPORTANT *******
************************

There is a bug in wp-trackback.php that will stop favatars from working
with trackbacks. Here's how to fix it:
				
	You must edit wp-trackback.php to enable this. 
	Switch line 88 and line 99:

		trackback_response(0);
		do_action('trackback_post', $comment_ID);

	becomes:
	
		do_action('trackback_post', $comment_ID);
		trackback_response(0);

		
	Additionally, you may want to comment 
	out line 65
	
		$author = addslashes($author);
	
	since it's just adding too many slashes 
	and messing things up.





***** USEAGE *****

Place:

	<?php comment_favicon(); ?>

Inside your comment loop (usually in wp-comments.php).

Options:
	comment_favicon($before='<img src="', $after='" alt="" />');

	The default function surrounds the image url with the before and after variables


*/

// Test to see if this hack has been loaded yet...
// If not, create the extra needed column in the comments table. 

$comment_favicon_exists = fav_maybe_add_column($tablecomments, 'comment_favicon_url', "ALTER TABLE `$tablecomments` ADD `comment_favicon_url` TEXT NOT NULL");

if( $comment_favicon_exists == false ) {
	die("I could not initialize the need column to make the favicon hack work. Please disable the plugin and contact the author.");
}



// Now the real meat, the discovery function.	

function discover_favicon($comment_ID) {
    global $wpdb, $tablecomments;

    $posters_url = $wpdb->get_var("SELECT comment_author_url FROM `$tablecomments` WHERE `comment_ID` = '$comment_ID'");

	if(!empty($posters_url)) { 

		if( $favicon_url = getFavicon($posters_url) ) {
			$wpdb->query("UPDATE `$tablecomments` SET `comment_favicon_url` = '$favicon_url' WHERE `comment_ID` = '$comment_ID' LIMIT 1");
		} 

	} 
	
	return $comment_ID;

}

// adds the discovery function to happen on comments, tracks, and pings
// PLEASE SEE THE NOTE ABOUT THE trackback_post BUG
add_action('comment_post','discover_favicon');
add_action('trackback_post','discover_favicon');
add_action('pingback_post','discover_favicon');

function comment_favicon($before='<img src="', $after='" alt="" />') {
	global $comment, $wpdb, $tablecomments;
    $favicon_url = $wpdb->get_var("SELECT comment_favicon_url FROM `$tablecomments` WHERE `comment_ID` = '$comment->comment_ID'");
	if(!empty($favicon_url)) { 
		echo($before . $favicon_url . $after);
	}
}





/**************************************
Utility functions needed for this thing
**************************************/

/*
* @return boolean/string
* @param  string $url
* @desc  Attempts to find and return a favicon url based on $url
*/

function getFavicon($url) {

	// start by fetching the contents of the URL they left...
	if( $html = @file_get_contents($url) ) {

		if (preg_match('/<link[^>]+rel="(?:shortcut )?icon"[^>]+?href="([^"]+?)"/si', $html, $matches)) {
			
			// Attempt to grab a favicon link from their webpage url

			$linkUrl = html_entity_decode($matches[1]);
			if (substr($linkUrl, 0, 1) == '/') {
				$urlParts = parse_url($url);
				$faviconURL = $urlParts['scheme'].'://'.$urlParts['host'].$linkUrl;
			} else if (substr($linkUrl, 0, 7) == 'http://') {
				$faviconURL = $linkUrl;
			} else if (substr($url, -1, 1) == '/') {
				$faviconURL = $url.$linkUrl;
			} else {
				$faviconURL = $url.'/'.$linkUrl;
			}

		} else {

			// If unsuccessful, attempt to "guess" the favicon location

			$urlParts = parse_url($url);
			$faviconURL = $urlParts['scheme'].'://'.$urlParts['host'].'/favicon.ico';

		}
		

		// Run a test to see if what we have attempted to get actually exists.

		if( $faviconURL_exists = url_validate($faviconURL) ) {
			return $faviconURL;
		}

	} 

	// Finally, if we haven't 'returned' yet then there is nothing to see here.
	return false;
}


/**
 ** maybe_add_column()
 ** Add column to db table if it doesn't exist.
 ** Returns:  true if already exists or on successful completion
 **           false on error
 */
function fav_maybe_add_column($table_name, $column_name, $create_ddl) {
    global $wpdb, $debug;
    foreach ($wpdb->get_col("DESC $table_name", 0) as $column ) {
        if ($debug) echo("checking $column == $column_name<br />");
        if ($column == $column_name) {
            return true;
        }
    }
    //didn't find it try to create it.
    $q = $wpdb->query($create_ddl);
    // we cannot directly tell that whether this succeeded!
    foreach ($wpdb->get_col("DESC $table_name", 0) as $column ) {
        if ($column == $column_name) {
            return true;
        }
    }
    return false;
}

/*
* @return boolean
* @param  string $link
* @desc  Checks to see if $link actually exists (HTTP-Code: 200|30*, etc)
*/
function url_validate( $link ) {  
		
	$url_parts = @parse_url( $link );

	if ( empty( $url_parts["host"] ) ) { return false; }

	if ( !empty( $url_parts["path"] ) ) {
		$documentpath = $url_parts["path"];
	} else {
		$documentpath = "/";
	}

	if ( !empty( $url_parts["query"] ) ) {
		$documentpath .= "?" . $url_parts["query"];
	}

	$host = $url_parts["host"];
	$port = $url_parts["port"];
	if ( empty($port) ) { $port = "80"; }

	$socket = @fsockopen( $host, $port, $errno, $errstr, 30 );
	
	if (!$socket) {
		return false;
	} else {
		fwrite ($socket, "HEAD ".$documentpath." HTTP/1.0\r\nHost: $host\r\n\r\n");
	
		$http_response = fgets( $socket, 22 );

		$responses = "/(200 OK)|(30[0-9] Moved)/";
		if ( preg_match($responses, $http_response) ) {
			return true;
			fclose($socket);
		} else {
			// echo "HTTP-Response: $http_response<br>";
			return false;
		}
	}

}
/*
Change Log:


0.1b		Release by Joen
0.2b		Recode and release by Jeff
0.2.1b		Removed an echo statement (line 187)
			Added an empty check for comment_favicon (line 70)
1.0			Added pingback and trackback detection. 
				HOWEVER
				You must edit wp-trackback.php to enable this. 
				Switch line 88 and line 99:

					trackback_response(0);
					do_action('trackback_post', $comment_ID);

				becomes:
				
					do_action('trackback_post', $comment_ID);
					trackback_response(0);

				Also, comment out line 65
				
					$author = addslashes($author);
				
				since it's just adding too many slashes 
				and messing things up.

			Added `@` to file_get_contents in the 
				getFavicon($url) function to silence
				404 errors.

			Added favatise.php to generate favatars for 
				all previous comments, tracks, and pings.

1.0.1		Added to wp-plugins.net so that I actually get
				change tracking :D
				
1.0.2		Updated a url, cleaed up favatise.php a bit



*/
?>